import {
  Datasource,
  IEventParam,
  IDictionary,
  Metadata,
  Http,
} from '@zeedhi/core';
import {
  Button,
  Grid,
  Select,
  Switch,
  GridEditable,
  Modal,
  ModalService,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class ParametersDashboardController {
  public departmentName: string = '';
  
  public align: string = '';
  
  public justify: string = '';
  
  public disabled: boolean = false;

  public cardIsVisible = false;
  
  public initialPanelValue: boolean = false;
  
  private fromExceptionOperatorsModal!: Modal;
  
  private fromOperatorsModal!: Modal;
  
  private fromProfileAccessModal!: Modal;
  
  private fromReplicationModal!: Modal;
  
  private fromAutomaticRegistrationModal!: Modal;

  get cardVisibility() {
    return !this.cardIsVisible;
  }

  private async deleteAndUpdate(datasource: Datasource, row: IDictionary<any>) {
    await datasource.delete(row);
    datasource.get();
  }
  
  private async enableOrDisableInitialPanel(datasource: Datasource, row: IDictionary<any>) {
    await datasource.delete(row);
    datasource.get();
  }
  
  private async enableOrDisableIndicators(datasource: Datasource, row: IDictionary<any>) {
    await datasource.delete(row);
    datasource.get();
  }
  
  public filter() {
    const grid = Metadata.getInstance<TekGrid>('parametersTekGrid');
    const [unity] = grid.getFilterInputs('CDFILIAL');
    const [panel] = grid.getFilterInputs('NMPAINEL');

    const conditions = {
      CDFILIAL: unity.value,
      NRPAINEL: panel.value,
    };
    
    grid.datasource.filter = conditions;
    grid.datasource.get();
  }

  public cellClick({ row, column, component }: IEventParam<Grid>) {
    const isDeleteAction = column.name === 'delete-action';
    const isSwitchAction = column.name === 'initial-painel-switch';
    if (isDeleteAction) {
      this.deleteAndUpdate(component.datasource, row);
      return isDeleteAction;
    }

    if (isSwitchAction) {
      this.enableOrDisableInitialPanel(component.datasource, row);
      return isSwitchAction;
    }

    return isDeleteAction;
  }

  public rowClick() {
    this.cardIsVisible = true;
  }

  public showProfileAccessCard() {
    this.cardIsVisible = false;
  }
  
  public testCellIsEditable({ row }: IDictionary<any>) {
    return parseFloat(row.id) % 3 !== 0;
  }

  public isNotInlineEditingConditions = true;

  public onInlineEditConditions({ component }: IEventParam<GridEditable>) {
    this.isNotInlineEditingConditions = !component.editing;
  }
  
  public async cancelEditingConditions({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.cancelEditedRows();
    this.onInlineEditConditions({ component: grid });
  }

  public async saveEditingConditions({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.saveEditedRows();
    this.onInlineEditConditions({ component: grid });
  }
  
  public async createExceptionOperatorsModal() {
    this.fromExceptionOperatorsModal = await ModalService.createFromJson('modals/addOperatorException', true);
  }

  public destroyExceptionOperatorsModal() {
    this.fromExceptionOperatorsModal?.destroy();
  }

  public openExceptionOperatorsModal() {
    this.fromExceptionOperatorsModal.show();
  }

  public closeExceptionOperatorsModal() {
    this.fromExceptionOperatorsModal.hide();
  }
  
  public async createOperatorsModal() {
    this.fromOperatorsModal = await ModalService.createFromJson('modals/indicatorsListToOperatorsException', true);
  }

  public destroyOperatorsModal() {
    this.fromOperatorsModal?.destroy();
  }

  public openOperatorsModal() {
    this.fromOperatorsModal.show();
  }

  public closeOperatorsModal() {
    this.fromOperatorsModal.hide();
  }
  
  public async createProfileAccessModal() {
    this.fromProfileAccessModal = await ModalService.createFromJson('modals/addProfileAccess', true);
  }

  public destroyProfileAccessModal() {
    this.fromProfileAccessModal?.destroy();
  }

  public openProfileAccessModal() {
    this.fromProfileAccessModal.show();
  }

  public closeProfileAccessModal() {
    this.fromProfileAccessModal.hide();
  }
  
  public async addProfileAccess() {
    const grid = Metadata.getInstance<Grid>('loadProfileAccessDataSourceModalHeader');
    const profileAccess = grid.getChildInstance<Select>('NMPERFILACESSO');
    const initialPanel = grid.getChildInstance<Switch>('IDPADRAODASH');
    const sendObject = {
      profileAccess: profileAccess.value.NRPERFILACESSO,
      initialPanel: initialPanel.value,
    };
    await Http.post('/addProfileAccess', sendObject);
  }
  
  public async createReplicationModal() {
    this.fromReplicationModal = await ModalService.createFromJson('modals/replication', true);
  }

  public destroyReplicationModal() {
    this.fromReplicationModal?.destroy();
  }

  public openReplicationModal() {
    this.fromReplicationModal.show();
  }

  public closeReplicationModal() {
    this.fromReplicationModal.hide();
  }
  
  public async createAutomaticRegistrationModal() {
    this.fromAutomaticRegistrationModal = await ModalService.createFromJson('modals/automaticRegistration', true);
  }

  public destroyAutomaticRegistrationModal() {
    this.fromAutomaticRegistrationModal?.destroy();
  }

  public openAutomaticRegistrationModal() {
    this.fromAutomaticRegistrationModal.show();
  }

  public closeAutomaticRegistrationModal() {
    this.fromAutomaticRegistrationModal.hide();
  }
}
